__author__ = 'saeedamen'  # Saeed Amen

#
# Copyright 2016-2020 Cuemacro - https://www.cuemacro.com / @cuemacro
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed
# under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
# CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#

import datetime

from findatapy.market import Market, MarketDataGenerator, MarketDataRequest
from finmarketpy.backtest import TradingModel, BacktestRequest
from finmarketpy.economics import TechIndicator

from chartpy import Style

from findatapy.util.dataconstants import DataConstants

# You will likely need to change this!
# Get an API key from the FRED website https://fred.stlouisfed.org/docs/api/api_key.html
fred_api_key = DataConstants().fred_api_key


class TradingModelFXTrend_Example(TradingModel):
    """Shows how to create a simple FX CTA style strategy, using the TradingModel abstract class (backtest_examples.py
    is a lower level way of doing this).
    """

    def __init__(self):
        super(TradingModel, self).__init__()

        ##### FILL IN WITH YOUR OWN PARAMETERS FOR display, dumping, TSF etc.
        self.market = Market(market_data_generator=MarketDataGenerator())
        self.DUMP_PATH = ""
        self.FINAL_STRATEGY = "FX trend"
        self.SCALE_FACTOR = 1
        self.DEFAULT_PLOT_ENGINE = "plotly"
        # self.CHART_STYLE = Style(plotly_plot_mode="offline_jupyter")

        self.br = self.load_parameters()
        return

    ###### Parameters and signal generations (need to be customised for every model)
    def load_parameters(self, br=None):

        if br is not None: return br

        ##### FILL IN WITH YOUR OWN BACKTESTING PARAMETERS
        br = BacktestRequest()

        # get all asset data
        br.start_date = "04 Jan 1989"
        br.finish_date = datetime.datetime.utcnow().date()
        br.spot_tc_bp = 0.5
        br.ann_factor = 252

        br.plot_start = "01 Jan 2000"
        br.calc_stats = True
        br.write_csv = False
        br.plot_interim = True
        br.include_benchmark = True

        # Have vol target for each signal
        br.signal_vol_adjust = True
        br.signal_vol_target = 0.1
        br.signal_vol_max_leverage = 5
        br.signal_vol_periods = 20
        br.signal_vol_obs_in_year = 252
        br.signal_vol_rebalance_freq = "BME"
        br.signal_vol_resample_freq = None

        # Have vol target for portfolio
        br.portfolio_vol_adjust = True
        br.portfolio_vol_target = 0.1
        br.portfolio_vol_max_leverage = 5
        br.portfolio_vol_periods = 20
        br.portfolio_vol_obs_in_year = 252
        br.portfolio_vol_rebalance_freq = "BME"
        br.portfolio_vol_resample_freq = None

        # Tech params
        br.tech_params.sma_period = 200

        # To make additive indices
        # br.cum_index = "add"

        return br

    def load_assets(self, br=None):
        ##### FILL IN WITH YOUR ASSET DATA
        from findatapy.util.loggermanager import LoggerManager
        logger = LoggerManager().getLogger(__name__)

        # For FX basket
        full_bkt = ["EURUSD", "USDJPY", "GBPUSD", "AUDUSD", "USDCAD",
                    "NZDUSD", "USDCHF", "USDNOK", "USDSEK"]

        basket_dict = {}

        for i in range(0, len(full_bkt)):
            basket_dict[full_bkt[i]] = [full_bkt[i]]

        basket_dict["FX trend"] = full_bkt

        br = self.load_parameters(br=br)

        logger.info("Loading asset data...")

        vendor_tickers = ["DEXUSEU", "DEXJPUS",
                          "DEXUSUK", "DEXUSAL",
                          "DEXCAUS",
                          "DEXUSNZ", "DEXSZUS",
                          "DEXNOUS", "DEXSDUS"]

        market_data_request = MarketDataRequest(
            start_date=br.start_date,  # start date
            finish_date=br.finish_date,  # finish date
            freq="daily",  # daily data
            data_source="alfred",  # use ALFRED/FRED as data source
            tickers=full_bkt,  # ticker (Cuemacro)
            fields=["close"],  # which fields to download
            vendor_tickers=vendor_tickers,  # ticker
            vendor_fields=["close"],  # which fields to download
            cache_algo="cache_algo_return",  # how to return data
            fred_api_key=fred_api_key)

        asset_df = self.market.fetch_market(market_data_request)

        # If web connection fails read from CSV
        if asset_df is None:
            import pandas

            asset_df = pandas.read_csv(
                "fxcta.csv", index_col=0, parse_dates=["Date"],
                date_parser=lambda x: pandas.datetime.strptime(x, "%Y-%m-%d"))

        # Signalling variables
        spot_df = asset_df
        spot_df2 = None

        # asset_df

        return asset_df, spot_df, spot_df2, basket_dict

    def construct_signal(self, spot_df, spot_df2, tech_params, br,
                         run_in_parallel=False):

        ##### FILL IN WITH YOUR OWN SIGNALS

        # Use technical indicator to create signals
        # (we could obviously create whatever function we wanted for generating the signal dataframe)
        tech_ind = TechIndicator()
        tech_ind.create_tech_ind(spot_df, "SMA", tech_params);
        signal_df = tech_ind.get_signal()

        return signal_df

    def construct_strategy_benchmark(self):

        ###### FILL IN WITH YOUR OWN BENCHMARK

        md_request = MarketDataRequest(
            start_date=self.br.start_date,  # start date
            finish_date=self.br.finish_date,  # finish date
            freq="daily",  # daily frequency
            data_source="alfred",  # use ALFRED/FRED as data source
            tickers=["EURUSD"],  # tickers to download
            vendor_tickers=["DEXUSEU"],
            fields=["close"],  # which fields to download
            vendor_fields=["close"],
            cache_algo="cache_algo_return",  # how to return data
            fred_api_key=fred_api_key)

        df = self.market.fetch_market(md_request)

        df.columns = [x.split(".")[0] for x in df.columns]

        return df


if __name__ == "__main__":

    # Just change "False" to "True" to run any of the below examples

    # Create a FX trend strategy then chart the returns, leverage over time
    if True:
        model = TradingModelFXTrend_Example()

        model.construct_strategy()

        model.plot_strategy_pnl()  # plot the final strategy
        model.plot_strategy_leverage()  # plot the leverage of the portfolio
        model.plot_strategy_group_pnl_trades()  # plot the individual trade P&Ls
        model.plot_strategy_group_benchmark_pnl()  # plot all the cumulative P&Ls of each component
        model.plot_strategy_group_benchmark_pnl_ir()  # plot all the IR of individual components
        model.plot_strategy_group_leverage()  # plot all the individual leverages

        from finmarketpy.backtest import TradeAnalysis

        ta = TradeAnalysis()

        # Create statistics for the model returns using both finmarketpy and pyfolio
        ta.run_strategy_returns_stats(model, engine="finmarketpy")
        # ta.run_strategy_returns_stats(model, engine="pyfolio")

        # model.plot_strategy_group_benchmark_annualised_pnl()

    # Create a FX CTA strategy, then examine how P&L changes with different vol targeting
    # and later transaction costs
    if True:
        strategy = TradingModelFXTrend_Example()

        from finmarketpy.backtest import TradeAnalysis

        ta = TradeAnalysis()
        ta.run_strategy_returns_stats(model, engine="finmarketpy")

        # Which backtesting parameters to change
        # names of the portfolio
        # broad type of parameter name
        parameter_list = [
            {"portfolio_vol_adjust": True, "signal_vol_adjust": True},
            {"portfolio_vol_adjust": False, "signal_vol_adjust": False}]

        pretty_portfolio_names = \
            ["Vol target",
             "No vol target"]

        parameter_type = "vol target"

        ta.run_arbitrary_sensitivity(strategy,
                                     parameter_list=parameter_list,
                                     pretty_portfolio_names=pretty_portfolio_names,
                                     parameter_type=parameter_type)

        # Now examine sensitivity to different transaction costs
        tc = [0, 0.25, 0.5, 0.75, 1, 1.25, 1.5, 1.75, 2.0]
        ta.run_tc_shock(strategy, tc=tc)

        # How does P&L change on day of month
        ta.run_day_of_month_analysis(strategy)

    # Create a FX CTA strategy then use TradeAnalysis (via pyfolio) to analyse returns
    if False:
        from finmarketpy.backtest import TradeAnalysis

        model = TradingModelFXTrend_Example()
        model.construct_strategy()

        tradeanalysis = TradeAnalysis()
        tradeanalysis.run_strategy_returns_stats(strategy)



__author__ = 'saeedamen'  # Saeed Amen

#
# Copyright 2016-2020 Cuemacro - https://www.cuemacro.com / @cuemacro
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed
# under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
# CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""
Examples for downloading economic data events from Bloomberg
"""

# for logging
import pandas
import pytz

from chartpy import Chart, Style

from findatapy.market import Market, MarketDataGenerator, MarketDataRequest
from findatapy.timeseries import Calculations
from findatapy.util import LoggerManager

from finmarketpy.economics import EventStudy

try:
    from finaddpy.market.cachedmarketdatagenerator import \
        CachedMarketDataGenerator as MarketDataGenerator
except:
    pass

# choose run_example = 0 for everything
# run_example = 1 - download recent NFP times and do event study for USD/JPY

run_example = 0

###### download recent NFP times and do event study for USD/JPY (using Bloomberg data)
if run_example == 1 or run_example == 0:
    logger = LoggerManager().getLogger(__name__)

    import datetime

    from datetime import timedelta

    ###### Get intraday data for USD/JPY from the past few months from Bloomberg, NFP date/times from Bloomberg
    ###### then plot intraday price action around NFP for EUR/USD

    start_date = datetime.date.today() - timedelta(days=180)
    finish_date = datetime.datetime.utcnow().date()

    market = Market(market_data_generator=MarketDataGenerator())

    # Fetch NFP times from Bloomberg
    md_request = MarketDataRequest(
        start_date=start_date,  # start date
        finish_date=finish_date,  # finish date
        category="events",
        freq='daily',  # daily data
        data_source='bloomberg',  # use Bloomberg as data source
        tickers=['NFP'],
        fields=['release-date-time-full'],  # which fields to download
        vendor_tickers=['NFP TCH Index'],  # ticker (Bloomberg)
        cache_algo='cache_algo_return')  # how to return data

    df_event_times = market.fetch_market(md_request)
    df_event_times = pandas.DataFrame(
        index=df_event_times['NFP.release-date-time-full'])

    # Need same timezone for event times as market data (otherwise can cause problems with Pandas)
    df_event_times = df_event_times.tz_localize('utc')

    # Fetch USD/JPY spot
    md_request = MarketDataRequest(
        start_date=start_date,  # start date
        finish_date=finish_date,  # finish date
        category='fx',
        freq='intraday',  # intraday
        data_source='bloomberg',  # use Bloomberg as data source
        tickers=['USDJPY'],  # ticker (finmarketpy)
        fields=['close'],  # which fields to download
        cache_algo='cache_algo_return')  # how to return data

    df = None
    df = market.fetch_market(md_request)

    calc = Calculations()
    df = calc.calculate_returns(df)

    es = EventStudy()

    # Work out cumulative asset price moves moves over the event
    df_event = es.get_intraday_moves_over_custom_event(df, df_event_times)

    # Create an average move
    df_event['Avg'] = df_event.mean(axis=1)

    # Plotting spot over economic data event
    style = Style()
    style.scale_factor = 3
    style.file_output = 'usdjpy-nfp.png'

    style.title = 'USDJPY spot moves over recent NFP'

    # Plot in shades of blue (so earlier releases are lighter, later releases are darker)
    style.color = 'Blues';
    style.color_2 = []
    style.y_axis_2_series = []
    style.display_legend = False
    style.auto_scale = True

    # Last release will be in red, average move in orange
    style.color_2_series = [df_event.columns[-2], df_event.columns[-1]]
    style.color_2 = ['red', 'orange']  # red, pink
    style.linewidth_2 = 2
    style.linewidth_2_series = style.color_2_series

    chart = Chart(engine='matplotlib')
    chart.plot(df_event * 100, style=style)



__author__ = 'saeedamen'  # Saeed Amen

#
# Copyright 2016-2020 Cuemacro - https://www.cuemacro.com / @cuemacro
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed
# under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
# CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#

import abc

class AbstractCurve(object):
    """Abstract class for creating total return indices and curves, which is for example implemented by FXSpotCurve
    and could be implemented by other asset classes.

    """

    @abc.abstractmethod
    def generate_key(self):
        return

    @abc.abstractmethod
    def fetch_continuous_time_series(self, md_request, market_data_generator):
        return

    @abc.abstractmethod
    def construct_total_returns_index(self):
        return



__author__ = 'saeedamen'  # Saeed Amen

#
# Copyright 2016-2020 Cuemacro - https://www.cuemacro.com / @cuemacro
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed
# under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
# CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#

# loading data
import datetime

from chartpy import Chart, Style
from findatapy.market import Market, MarketDataGenerator, MarketDataRequest
from finmarketpy.economics import TechIndicator, TechParams

from findatapy.util.loggermanager import LoggerManager

logger = LoggerManager().getLogger(__name__)

chart = Chart(engine="matplotlib")

market = Market(market_data_generator=MarketDataGenerator())
tech_ind = TechIndicator()

# Choose run_example = 0 for everything
# run_example = 1 - download S&P500 from Bloomberg, calculate ATR and plot

run_example = 0

###### Fetch data from Bloomberg for S&P500
if run_example == 1 or run_example == 0:
    # Downloaded S&P500
    md_request = MarketDataRequest(
        start_date="01 Jan 2000",  # start date
        data_source="bloomberg",  # use Bloomberg as data source
        tickers=["S&P500"],
        fields=["close", "open", "high", "low"],  # which fields to download
        vendor_tickers=["SPX Index"],  # ticker (Bloomberg)
        vendor_fields=["PX_LAST", "PX_OPEN", "PX_HIGH", "PX_LOW"],
        # which Bloomberg fields to download
        cache_algo="internet_load_return")  # how to return data

    df = market.fetch_market(md_request)

    print(df)

    tech_params = TechParams()
    tech_params.atr_period = 14
    tech_ind.create_tech_ind(df, "ATR", tech_params)

    style = Style()

    style.title = "S&P500 ATR"
    style.scale_factor = 2
    style.file_output = "sp500.png"
    style.source = "Bloomberg"

    df = tech_ind.get_techind()

    chart.plot(df, style=style)
